library(stockassessment)
load("run/model.RData")
###############################
#
# 
#  
##############################
#  ~~~~~~~~~~~~~Reference points~~~~~~~~~~~~~~~~~~~~~
# WKDEM 2020
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
F_MSY <- 0.3
F_MSY_lower <- 0.18
F_MSY_upper <- 0.49 
Fpa <- 0.57 #  New Fpa = Fp05 from WKDEM 2020, previously Fpa=0.52
Flim <- 0.73
Blim <- 14376
Bpa <- 20126
MSY_Btrigger <- 20126

# TAC & advice for 2021

prev.advice <- 0 # not used
prev.TAC <-1279 #this was 2021 & 2022

#2021 catch
prev.cat <-1610 # not used

NSIM <-10000

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  FORECAST ASSUMPTIONS
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Modified WGCSE 2022
#  - No intermediate year in the assessment
#  - exclusion of age 1 & age 2 discard info (catch mean wts, stock mean wts
#   landfrac) in final assessment year from 3 year average - fix is to replace
#   final year with average of previous 2

# ---- Final year in the assessment
final.yr <-max(fit$data$years)

# ---- Resample R from previous 6 years Rec - shortened period
R.range <-(final.yr-5):final.yr

# ---- Use three year average for mean weights & L-D partition (see above/below for age 1 & 2 fix)
av.yrs <-(final.yr-2):final.yr

#----- Use 3 year average selection patter
sel.yrs <-(final.yr-2):final.yr

# ---- F status quo is 3 year average
f_sq <-mean(fbartable(fit)[as.character(sel.yrs),"Estimate"])

#----- Final assessment year F - needed to run the forecast through 2021 to get numbers/SSB at start of intermediate year
f_final <-fbartable(fit)[as.character(final.yr),"Estimate"]

fit.save <-fit

ags <-1:2
rep.yrs <-c((final.yr-2),(final.yr-1))
fit$data$landFrac[as.character(final.yr),ags] <-colMeans(fit$data$landFrac[as.character(rep.yrs),ags])
fit$data$catchMeanWeight[as.character(final.yr),ags] <-colMeans(fit$data$catchMeanWeight[as.character(rep.yrs),ags])
fit$data$stockMeanWeight[as.character(final.yr),ags] <-colMeans(fit$data$stockMeanWeight[as.character(rep.yrs),ags])
fit$data$natMor[as.character(final.yr),ags] <-colMeans(fit$data$natMor[as.character(rep.yrs),ags])
#  No requirement to adjust discMeanWeight as not used in the forecast (Total disc = Catch - Land)


#########  dummy forecast   ##########
# 2022F=Fsq then F_MSY - run a dummy forecast to get the SSB at start of TAC year
set.seed(12345)
dummy.forecast <- forecast(fit, fscale=c(NA,NA,NA,NA), fval=c(f_final,f_sq,F_MSY,F_MSY), year.base = final.yr,
                               label="2022F=Fsq then zero", rec.years=R.range, ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE,nosim=NSIM,savesim=TRUE)

#  forecast proper
SSBnext <-attributes(dummy.forecast)$shorttab[3,3]

FC<-list()

scen <- list(
  "MSY approach: F_MSY" = list(fval = c(f_final, f_sq, F_MSY, F_MSY)),
  "Precautionary approach: Fpa" = list(fval = c(f_final,f_sq, Fpa, Fpa)),
  "FMSY upper" = list(fval = c(f_final, f_sq, F_MSY_upper, F_MSY_upper)),
  "FMSY lower" = list(fval = c(f_final, f_sq, F_MSY_lower, F_MSY_lower)),
  "F = 0" = list(fval = c(f_final, f_sq, NA, NA),fscale=c(NA,NA,0.0,0.0)),
  "Fpa" = list(fval = c(f_final,f_sq, Fpa, Fpa)),
  "F = Flim" = list(fval = c(f_final, f_sq, Flim, Flim)),
  "Fsq"  = list(fval = c(f_final, f_sq, f_sq, f_sq)),
  "zero TAC advice - haddock Fmult" = list(fval = c(f_final, f_sq, 2.16*f_sq,2.16*f_sq)),
  "zero TAC advice - saith Fmult" = list(fval = c(f_final, f_sq, 1.0*f_sq,1.0*f_sq)),
  "0.05*Fsq" = list(fval = c(f_final, f_sq, 0.05*f_sq,0.05*f_sq)),
  "0.25*Fsq" = list(fval = c(f_final, f_sq, 0.25*f_sq,0.25*f_sq)),
  "0.5*Fsq" = list(fval = c(f_final, f_sq, 0.5*f_sq,0.5*f_sq)),
  "0.75*Fsq" = list(fval = c(f_final, f_sq, 0.75*f_sq,0.75*f_sq))
)

FC <- list()
#FC <- vector("list", length(scen))
#names(FC) <- names(scen)


for(i in seq(scen)){
  set.seed(23456)
  ARGS <- scen[[i]]
  ARGS <- c(ARGS,
            list(fit = fit, ave.years=av.yrs, rec.years=R.range, year.base=final.yr,label=names(scen)[i], 
                 overwriteSelYears = sel.yrs, splitLD = TRUE,
                 nosim=NSIM,savesim=TRUE))  
  FC[[i]] <- do.call(forecast, ARGS)
  
  print(paste0("forecast : ", "'", names(scen)[i], "'", " is complete"))
}


#  ICES advice rule for MSY - This seems overly complicated....
#  Not sure why can't just set fval to FMSY&SSBnext/MSY_Btrigger & run standard forecast
MSYappr <- function(fit, Fmsy=NULL, Btrig=NULL, fscale_init=1, fval_init=NA, catchval_init=NA, label=NA){
  
  fscale <- fscale_init
  fval <- fval_init
  catchval <- catchval_init
  
  for (i in 3:4){
    
    fscale[i] <- NA
    fval[i] <- Fmsy
    catchval[i] <- NA
    
    set.seed(23456)
    f1 <- forecast(fit, fscale=fscale, fval=fval, catchval=catchval, ave.years=av.yrs, rec.years=R.range, overwriteSelYears=sel.yrs)
    SSB <- attr(f1,"shorttab")[3,i]
    fval[i] <- min(Fmsy*SSB/Btrig, Fmsy)
    
  }
  
  set.seed(23456)
  f_final <- forecast(fit, fscale=fscale, fval=fval, catchval=catchval, ave.years=av.yrs, rec.years=R.range, label=label, overwriteSelYears=sel.yrs, splitLD=TRUE,nosim=NSIM,savesim=TRUE)
  return(f_final)
}
FC[[length(FC)+1]] <- MSYappr(fit, Fmsy=F_MSY, Btrig=MSY_Btrigger, fscale_init = NA, fval_init=c(f_final,f_sq), catchval_init = NA, label="2022F=Fsq then Fmsy HCR")

FC[[length(FC)+1]] <- MSYappr(fit, Fmsy=F_MSY_lower, Btrig=MSY_Btrigger, fscale_init = NA, fval_init=c(f_final,f_sq), catchval_init = NA, label="2022F=Fsq then Fmsy HCR lower")

FC[[length(FC)+1]] <- MSYappr(fit, Fmsy=F_MSY_upper, Btrig=MSY_Btrigger, fscale_init = NA, fval_init=c(f_final,f_sq), catchval_init = NA, label="2022F=Fsq then Fmsy HCR upper")


#SSB target
# X% increase
incr <-10
inc <-seq(0,50,incr)/100

for (i in 1:length(inc)){
  SSBtarget <-SSBnext*(1+inc[i])
  set.seed(23456)
  FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA,NA), fval=c(f_final,f_sq,NA,f_sq),nextssb=c(NA,NA,SSBtarget,NA),year.base = final.yr,
                               label=paste0("2022F=Fsq then ",(i-1)*incr,"% SSB increase"),rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE,nosim=NSIM,savesim=TRUE)

  
}

#  TAC reduction
# X% increase

inc <-seq(0.25,1,0.25)

for (i in 1:length(inc)){
  Catchtarget <-prev.TAC*inc[i]
  set.seed(23456)
  FC[[length(FC)+1]] <- forecast(fit, fval=c(f_final,f_sq,NA,NA), catchval=c(NA,NA,prev.TAC*inc[i],prev.TAC*inc[i]), year.base = final.yr,
                                                                label=paste0("2022F=Fsq then prev.TAC*",inc[i]), rec.years=R.range,ave.years = av.yrs,
                                                               overwriteSelYears = sel.yrs,splitLD = TRUE,nosim=NSIM,savesim=TRUE)
  
}  
  
nms <-names(scen)
LM <-length(scen)
for (i in (LM+1):length(FC)){
  nms <-c(nms,attr(FC[[i]],"label"))
}

names(FC) <-nms



save(FC, file="run/forecast.RData")
